Class: MaintenanceTasks::Run Private
- Inherits:
-
ApplicationRecord
- Object
- ActiveRecord::Base
- ApplicationRecord
- MaintenanceTasks::Run
- Defined in:
- app/models/maintenance_tasks/run.rb
Overview
This class is part of a private API. You should avoid using this class if possible, as it may be removed or be changed in the future.
Model that persists information related to a task being run from the UI.
Constant Summary collapse
- STATUSES =
This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.
Various statuses a run can be in.
[ :enqueued, # The task has been enqueued by the user. :running, # The task is being performed by a job worker. :succeeded, # The task finished without error. :cancelling, # The task has been told to cancel but is finishing work. :cancelled, # The user explicitly halted the task's execution. :interrupted, # The task was interrupted by the job infrastructure. :pausing, # The task has been told to pause but is finishing work. :paused, # The task was paused in the middle of the run by the user. :errored, # The task code produced an unhandled exception. ]
- ACTIVE_STATUSES =
This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.
[ :enqueued, :running, :paused, :pausing, :cancelling, :interrupted, ]
- STOPPING_STATUSES =
This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.
[ :pausing, :cancelling, ]
- COMPLETED_STATUSES =
This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.
[:succeeded, :errored, :cancelled]
- COMPLETED_RUNS_LIMIT =
This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.
10- STUCK_TASK_TIMEOUT =
This constant is part of a private API. You should avoid using this constant if possible, as it may be removed or be changed in the future.
5.minutes
Instance Method Summary collapse
-
#active? ⇒ Boolean
private
Returns whether the Run is active, which is defined as having a status of enqueued, running, pausing, cancelling, paused or interrupted.
-
#cancel ⇒ Object
private
Cancels a Run.
-
#complete ⇒ Object
private
Handles the completion of a Run, setting a status of succeeded and the ended_at timestamp.
-
#completed? ⇒ Boolean
private
Returns whether the Run is completed, which is defined as having a status of succeeded, cancelled, or errored.
-
#csv_attachment_presence ⇒ Object
private
Performs validation on the presence of a :csv_file attachment.
-
#csv_file ⇒ ActiveStorage::Attached::One
private
Fetches the attached ActiveStorage CSV file for the run.
-
#enqueued! ⇒ Object
private
Sets the run status to enqueued, making sure the transition is validated in case it’s already enqueued.
-
#job_shutdown ⇒ Object
private
Handles transitioning the status on a Run when the job shuts down.
-
#pausing! ⇒ Object
private
Marks a Run as pausing.
-
#persist_error(error) ⇒ Object
private
Marks the run as errored and persists the error data.
-
#persist_progress(number_of_ticks, duration) ⇒ Object
private
Increments
tick_countbynumber_of_ticksandtime_runningbyduration, both directly in the DB. -
#persist_transition ⇒ Object
private
Saves the run, persisting the transition of its status, and all other changes to the object.
-
#reload_status ⇒ MaintenanceTasks::Run
private
Refreshes the status and lock version attributes on the Active Record object, and ensures ActiveModel::Dirty doesn’t mark the object as changed.
-
#running ⇒ Object
private
Marks a Run as running.
-
#start(count) ⇒ Object
private
Starts a Run, setting its started_at timestamp and tick_total.
-
#started? ⇒ Boolean
private
Returns whether the Run has been started, which is indicated by the started_at timestamp being present.
-
#stopped? ⇒ Boolean
private
Returns whether the Run is stopped, which is defined as having a status of paused, succeeded, cancelled, or errored.
-
#stopping? ⇒ Boolean
private
Returns whether the Run is stopping, which is defined as having a status of pausing or cancelled.
-
#stuck? ⇒ Boolean
private
Returns whether a Run is stuck, which is defined as having a status of cancelling, and not having been updated in the last 5 minutes.
-
#task ⇒ Task
private
Returns a Task instance for this Run.
-
#time_to_completion ⇒ ActiveSupport::Duration
private
Returns the duration left for the Run to finish based on the number of ticks left and the average time needed to process a tick.
-
#validate_task_arguments ⇒ Object
private
Performs validation on the arguments to use for the Task.
Instance Method Details
#active? ⇒ Boolean
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Returns whether the Run is active, which is defined as having a status of enqueued, running, pausing, cancelling, paused or interrupted.
208 209 210 |
# File 'app/models/maintenance_tasks/run.rb', line 208 def active? ACTIVE_STATUSES.include?(status.to_sym) end |
#cancel ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Cancels a Run.
If the Run is paused, it will transition directly to cancelled, since the Task is not being performed. In this case, the ended_at timestamp will be updated.
If the Run is not paused, the Run will transition to cancelling.
If the Run is already cancelling, and has last been updated more than 5 minutes ago, it will transition to cancelled, and the ended_at timestamp will be updated.
298 299 300 301 302 303 304 305 306 307 308 309 |
# File 'app/models/maintenance_tasks/run.rb', line 298 def cancel if paused? || stuck? self.status = :cancelled self.ended_at = Time.now persist_transition else cancelling! end rescue ActiveRecord::StaleObjectError reload_status retry end |
#complete ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Handles the completion of a Run, setting a status of succeeded and the ended_at timestamp.
282 283 284 285 |
# File 'app/models/maintenance_tasks/run.rb', line 282 def complete self.status = :succeeded self.ended_at = Time.now end |
#completed? ⇒ Boolean
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Returns whether the Run is completed, which is defined as having a status of succeeded, cancelled, or errored.
199 200 201 |
# File 'app/models/maintenance_tasks/run.rb', line 199 def completed? COMPLETED_STATUSES.include?(status.to_sym) end |
#csv_attachment_presence ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Performs validation on the presence of a :csv_file attachment. A Run for a Task that uses CsvCollection must have an attached :csv_file to be valid. Conversely, a Run for a Task that doesn’t use CsvCollection should not have an attachment to be valid. The appropriate error is added if the Run does not meet the above criteria.
335 336 337 338 339 340 341 342 343 |
# File 'app/models/maintenance_tasks/run.rb', line 335 def if Task.named(task_name).has_csv_content? && !csv_file.attached? errors.add(:csv_file, "must be attached to CSV Task.") elsif !Task.named(task_name).has_csv_content? && csv_file.present? errors.add(:csv_file, "should not be attached to non-CSV Task.") end rescue Task::NotFoundError nil end |
#csv_file ⇒ ActiveStorage::Attached::One
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Fetches the attached ActiveStorage CSV file for the run. Checks first whether the ActiveStorage::Attachment table exists so that we are compatible with apps that are not using ActiveStorage.
386 387 388 389 390 391 |
# File 'app/models/maintenance_tasks/run.rb', line 386 def csv_file return unless defined?(ActiveStorage) return unless ActiveStorage::Attachment.table_exists? super end |
#enqueued! ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Sets the run status to enqueued, making sure the transition is validated in case it’s already enqueued.
Rescues and retries status transition if an ActiveRecord::StaleObjectError is encountered.
73 74 75 76 77 78 79 |
# File 'app/models/maintenance_tasks/run.rb', line 73 def enqueued! status_will_change! super rescue ActiveRecord::StaleObjectError reload_status retry end |
#job_shutdown ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Handles transitioning the status on a Run when the job shuts down.
269 270 271 272 273 274 275 276 277 278 |
# File 'app/models/maintenance_tasks/run.rb', line 269 def job_shutdown if cancelling? self.status = :cancelled self.ended_at = Time.now elsif pausing? self.status = :paused else self.status = :interrupted end end |
#pausing! ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Marks a Run as pausing.
Rescues and retries status transition if an ActiveRecord::StaleObjectError is encountered.
315 316 317 318 319 320 |
# File 'app/models/maintenance_tasks/run.rb', line 315 def pausing! super rescue ActiveRecord::StaleObjectError reload_status retry end |
#persist_error(error) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Marks the run as errored and persists the error data.
131 132 133 134 135 136 137 138 139 140 141 142 143 144 |
# File 'app/models/maintenance_tasks/run.rb', line 131 def persist_error(error) self.started_at ||= Time.now update!( status: :errored, error_class: truncate(:error_class, error.class.name), error_message: truncate(:error_message, error.), backtrace: MaintenanceTasks.backtrace_cleaner.clean(error.backtrace), ended_at: Time.now, ) run_task_callbacks(:error) rescue ActiveRecord::StaleObjectError reload_status retry end |
#persist_progress(number_of_ticks, duration) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Increments tick_count by number_of_ticks and time_running by duration, both directly in the DB. The attribute values are not set in the current instance, you need to reload the record.
114 115 116 117 118 119 120 121 122 123 124 125 126 |
# File 'app/models/maintenance_tasks/run.rb', line 114 def persist_progress(number_of_ticks, duration) self.class.update_counters( id, tick_count: number_of_ticks, time_running: duration, touch: true ) if locking_enabled? locking_column = self.class.locking_column self[locking_column] += 1 clear_attribute_change(locking_column) end end |
#persist_transition ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Saves the run, persisting the transition of its status, and all other changes to the object.
91 92 93 94 95 96 97 98 99 100 101 102 103 104 |
# File 'app/models/maintenance_tasks/run.rb', line 91 def persist_transition save! callback = CALLBACKS_TRANSITION[status] run_task_callbacks(callback) if callback rescue ActiveRecord::StaleObjectError success = succeeded? reload_status if success self.status = :succeeded else job_shutdown end retry end |
#reload_status ⇒ MaintenanceTasks::Run
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Refreshes the status and lock version attributes on the Active Record object, and ensures ActiveModel::Dirty doesn’t mark the object as changed.
This allows us to get the Run’s most up-to-date status without needing to reload the entire record.
153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 |
# File 'app/models/maintenance_tasks/run.rb', line 153 def reload_status columns_to_reload = if locking_enabled? [:status, self.class.locking_column] else [:status] end updated_status, updated_lock_version = self.class.uncached do self.class.where(id: id).pluck(*columns_to_reload).first end self.status = updated_status if updated_lock_version self[self.class.locking_column] = updated_lock_version end clear_attribute_changes(columns_to_reload) self end |
#running ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Marks a Run as running.
If the run is stopping already, it will not transition to running. Rescues and retries status transition if an ActiveRecord::StaleObjectError is encountered.
232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 |
# File 'app/models/maintenance_tasks/run.rb', line 232 def running if locking_enabled? begin running! unless stopping? rescue ActiveRecord::StaleObjectError reload_status retry end else # Preserve swap-and-replace solution for data races until users # run migration to upgrade to optimistic locking solution return if stopping? updated = self.class.where(id: id).where.not(status: STOPPING_STATUSES) .update_all(status: :running, updated_at: Time.now) > 0 if updated self.status = :running clear_attribute_changes([:status]) else reload_status end end end |
#start(count) ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Starts a Run, setting its started_at timestamp and tick_total.
260 261 262 263 264 265 266 |
# File 'app/models/maintenance_tasks/run.rb', line 260 def start(count) update!(started_at: Time.now, tick_total: count) run_task_callbacks(:start) rescue ActiveRecord::StaleObjectError reload_status retry end |
#started? ⇒ Boolean
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Returns whether the Run has been started, which is indicated by the started_at timestamp being present.
191 192 193 |
# File 'app/models/maintenance_tasks/run.rb', line 191 def started? started_at.present? end |
#stopped? ⇒ Boolean
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Returns whether the Run is stopped, which is defined as having a status of paused, succeeded, cancelled, or errored.
183 184 185 |
# File 'app/models/maintenance_tasks/run.rb', line 183 def stopped? completed? || paused? end |
#stopping? ⇒ Boolean
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Returns whether the Run is stopping, which is defined as having a status of pausing or cancelled.
175 176 177 |
# File 'app/models/maintenance_tasks/run.rb', line 175 def stopping? STOPPING_STATUSES.include?(status.to_sym) end |
#stuck? ⇒ Boolean
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Returns whether a Run is stuck, which is defined as having a status of cancelling, and not having been updated in the last 5 minutes.
326 327 328 |
# File 'app/models/maintenance_tasks/run.rb', line 326 def stuck? cancelling? && updated_at <= STUCK_TASK_TIMEOUT.ago end |
#task ⇒ Task
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Returns a Task instance for this Run. Assigns any attributes to the Task based on the Run’s parameters. Note that the Task instance is not supplied with :csv_content yet if it’s a CSV Task. This is done in the job, since downloading the CSV file can take some time.
399 400 401 402 403 404 405 406 407 408 409 |
# File 'app/models/maintenance_tasks/run.rb', line 399 def task @task ||= begin task = Task.named(task_name).new if task.attribute_names.any? && arguments.present? task.assign_attributes(arguments) end task rescue ActiveModel::UnknownAttributeError task end end |
#time_to_completion ⇒ ActiveSupport::Duration
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Returns the duration left for the Run to finish based on the number of ticks left and the average time needed to process a tick. Returns nil if the Run is completed, or if tick_count or tick_total is zero.
218 219 220 221 222 223 224 225 |
# File 'app/models/maintenance_tasks/run.rb', line 218 def time_to_completion return if completed? || tick_count == 0 || tick_total.to_i == 0 processed_per_second = (tick_count.to_f / time_running) ticks_left = (tick_total - tick_count) seconds_to_finished = ticks_left / processed_per_second seconds_to_finished.seconds end |
#validate_task_arguments ⇒ Object
This method is part of a private API. You should avoid using this method if possible, as it may be removed or be changed in the future.
Performs validation on the arguments to use for the Task. If the Task is invalid, the errors are added to the Run.
350 351 352 353 354 355 356 357 358 359 360 361 362 |
# File 'app/models/maintenance_tasks/run.rb', line 350 def validate_task_arguments arguments_match_task_attributes if arguments.present? if task.invalid? = task.errors .map { |attribute, | "#{attribute.inspect} #{message}" } errors.add( :arguments, "are invalid: #{error_messages.join("; ")}" ) end rescue Task::NotFoundError nil end |